import argparse
import os

from ncdu_dropbox.dirtree import DirTreeBuilder
from ncdu_dropbox.dropboxfetcher import DropboxFetcher
from ncdu_dropbox.ncduwriter import NcduWriter


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('path', nargs='?', default='', help='Path of directory to scan. Leave empty for entire Dropbox.')
    parser.add_argument('-t', '--token', help='Dropbox API Token. You can also provide the token by setting '
                                        'the DROPBOX_TOKEN environment variable.')
    parser.add_argument('-o', '--output', dest='writer', metavar='FILE', help='Write output to FILE.',
                        type=argparse.FileType('w'), default='-')
    args = parser.parse_args()

    token = args.token or os.environ.get('DROPBOX_TOKEN')
    if not token:
        exit(parser.print_help())

    path = args.path
    if path and not path.startswith('/'):
        path = '/' + path

    fetcher = DropboxFetcher(token)
    entries = fetcher.list(path)

    builder = DirTreeBuilder(path)
    dir_tree = builder.build(entries)

    exporter = NcduWriter(args.writer)
    exporter.export(dir_tree)


if __name__ == '__main__':
    main()


import json
import time

from ncdu_dropbox.dirtree import FileRecord


class NcduWriter:
    def __init__(self, writer):
        self.writer = writer

    def export(self, dir_tree):
        """Exports the directory tree to ncdu compatible file format."""
        ncdu_dict = export_to_ncdu_dict(dir_tree)
        ncdu_json = [1, 0,
                     {'progname': 'ncdu-dropbox', 'progver': '0.1', 'timestamp': int(time.time())},
                     ncdu_dict]
        json.dump(ncdu_json, self.writer)


def export_to_ncdu_dict(record):
    if isinstance(record, FileRecord):
        return {'name': record.name, 'dsize': record.size}
    else:
        return [{'name': record.name}] + [export_to_ncdu_dict(child) for child in record.children.values()]




import dropbox
import os


class DirRecord:
    def __init__(self, name):
        self.name = name
        self.children = {}

    def __repr__(self):
        return 'Directory({})'.format(self.name)


class FileRecord:
    def __init__(self, name, size):
        self.name = name
        self.size = size

    def __repr__(self):
        return 'File({}, {})'.format(self.name, self.size)


def make_record(entry):
    if isinstance(entry, dropbox.files.FolderMetadata):
        return DirRecord(entry.name)
    return FileRecord(entry.name, entry.size)


class DirTreeBuilder:
    def __init__(self, path):
        if not path:
            path = "/"
        self.root_path = path
        self.dir_tree = DirRecord(path)

    def build(self, entries):
        for entry in entries:
            record = make_record(entry)
            self.insert_record(entry.path_display, record)
        return self.dir_tree

    def insert_record(self, path, record):
        walk = self.walk_from_root(path)
        cur_dict = self.dir_tree.children
        for directory in walk[:-1]:
            key = directory.lower()
            if key not in cur_dict:
                cur_dict[key] = DirRecord(directory)
            cur_dict = cur_dict[key].children
        key = walk[-1].lower()
        if key not in cur_dict:
            cur_dict[key] = record
        else:
            cur_dict[key].name = record.name

    def walk_from_root(self, path):
        walk = []
        while path != self.root_path:
            path, cur = os.path.split(path)
            walk.append(cur)
        walk.reverse()
        return walk


from setuptools import setup, find_packages


def readme():
    with open('README.rst') as f:
        return f.read()


setup(
    name='ncdu-dropbox',
    version='0.1',
    description='Generates ncdu compatible json file for Dropbox',
    long_description=readme(),
    url='https://github.com/shitikanth/ncdu-dropbox',
    author='Shitikanth Kashyap',
    author_email='shitikanth1@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Topic :: Utilities',
    ],
    packages=find_packages(),
    install_requires=['dropbox'],
    entry_points={
        'console_scripts': [
            'ncdu-dropbox=ncdu_dropbox.main:main'
        ]
    }

)



import dropbox


class DropboxFetcher:
    def __init__(self, token):
        self.dbx = dropbox.Dropbox(token)

    def list(self, path):
        """Get a listing of all files and folders in Dropbox."""
        listing = self.dbx.files_list_folder(path=path, recursive=True)
        for entry in listing.entries:
            yield entry
        while listing.has_more:
            cursor = listing.cursor
            listing = self.dbx.files_list_folder_continue(cursor=cursor)
            for entry in listing.entries:
                yield entry